#!/bin/bash

# Shell script which is executed by bash *BEFORE* installation is started
# (*BEFORE* preinstall and *BEFORE* preupdate). Use with caution and remember,
# that all systems may be different!
#
# Exit code must be 0 if executed successfull. 
# Exit code 1 gives a warning but continues installation.
# Exit code 2 cancels installation.
#
# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# Will be executed as user "root".
# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#
# You can use all vars from /etc/environment in this script.
#
# We add 5 additional arguments when executing this script:
# command <TEMPFOLDER> <NAME> <FOLDER> <VERSION> <BASEFOLDER>
#
# For logging, print to STDOUT. You can use the following tags for showing
# different colorized information during plugin installation:
#
# <OK> This was ok!"
# <INFO> This is just for your information."
# <WARNING> This is a warning!"
# <ERROR> This is an error!"
# <FAIL> This is a fail!"

# To use important variables from command line use the following code:
COMMAND=$0    # Zero argument is shell command
PTEMPDIR=$1   # First argument is temp folder during install
PSHNAME=$2    # Second argument is Plugin-Name for scipts etc.
PDIR=$3       # Third argument is Plugin installation folder
PVERSION=$4   # Forth argument is Plugin version
#LBHOMEDIR=$5 # Comes from /etc/environment now. Fifth argument is
              # Base folder of LoxBerry
PTEMPPATH=$6  # Sixth argument is full temp path during install (see also $1)

# Combine them with /etc/environment
PHTML=$LBPHTML/$PDIR
PHTMLAUTH=$LBPHTMLAUTH/$PDIR
PTEMPL=$LBPTEMPL/$PDIR
PDATA=$LBPDATA/$PDIR
PLOG=$LBPLOG/$PDIR # Note! This is stored on a Ramdisk now!
PCONFIG=$LBPCONFIG/$PDIR
PBIN=$LBPBIN/$PDIR

#. $LBHOMEDIR/libs/bashlib/loxberry_log.sh
#PACKAGE=${PSHNAME}
#NAME=preroot_install
#FILENAME=${LBPLOG}/${PSHNAME}/preroot_install.log
#APPEND=1
#STDERR=1
  
echo "<INFO> Installation as root user started."

echo "<INFO> Start installing Python packages..."
yes | pip3 install requests

INSTALLED_WS=$(pip3 list --format=columns | grep "requests" | grep -v grep | wc -l)
if [ ${INSTALLED_WS} -ne "0" ]; then
	echo "<OK> Python requests installed successfully."
else
	echo "<WARNING> Python requests installation failed! The plugin will not work without."
	echo "<WARNING> Giving up."
	exit 2;
fi 

yes | pip3 install paho-mqtt

INSTALLED_WS=$(pip3 list --format=columns | grep "paho-mqtt" | grep -v grep | wc -l)
if [ ${INSTALLED_WS} -ne "0" ]; then
	echo "<OK> Python paho-mqtt installed successfully."
else
	echo "<WARNING> Python paho-mqtt installation failed! The plugin will not work without."
	echo "<WARNING> Giving up."
	exit 2;
fi 

yes | pip3 install influxdb

INSTALLED_WS=$(pip3 list --format=columns | grep "influxdb" | grep -v grep | wc -l)
if [ ${INSTALLED_WS} -ne "0" ]; then
	echo "<OK> Python influxdb installed successfully."
else
	echo "<WARNING> Python influxdb installation failed! The plugin will not work without."
	echo "<WARNING> Giving up."
	exit 2;
fi 

echo "<INFO> copy the service-startup-script to /etc/systemd/system/ and enable this daemon..."
chmod ugo-x ${PBIN}/foshkplugin.service
echo "cp -fp ${PBIN}/foshkplugin.service /etc/systemd/system/$PSHNAME.service"
cp -fp ${PBIN}/foshkplugin.service /etc/systemd/system/$PSHNAME.service
systemctl import-environment

echo "<INFO> set executable to ${PDATA}/foshkplugin.py..."
chmod 711 -c ${PDATA}/foshkplugin.py

echo "<INFO> creating config-file via autoConfig"
sudo -u loxberry ${PDATA}/foshkplugin.py -autoConfig $PSHNAME

echo "<INFO> Restarting the $PSHNAME-service"
$PBIN/service.sh daemon-reload
$PBIN/service.sh repair
$PBIN/service.sh restart

#echo "<INFO> Command is: $COMMAND"
#echo "<INFO> (Short) Name is: $PSHNAME"
#echo "<INFO> Installation folder is: $PDIR"
#echo "<INFO> Plugin version is: $PVERSION"
#echo "<INFO> Plugin HTML folder is: $PHTML"
#echo "<INFO> Plugin HTMLAUTH folder is: $PHTMLAUTH"
#echo "<INFO> Plugin Template folder is: $PTEMPL"
#echo "<INFO> Plugin Data folder is: $PDATA"
#echo "<INFO> Plugin Log folder (on RAMDISK!) is: $PLOG"
#echo "<INFO> Plugin CONFIG folder is: $PCONFIG"
#echo "<INFO> Plugin BIN folder is: $PBIN"
#echo "<INFO> PTEMPDIR is $PTEMPDIR"
#echo "<INFO> PTEMPPATH is $PTEMPPATH"

exit 0
