#!/usr/bin/perl
# This was a sample Script file
# It did not much:
#   * Loading configuration
#   * including header.htmlfooter.html
#   * and showing a message to the user.
# That's all.
# Now it does a bit more ...
# Oliver Engel; 15.12.2019, 28.12.2019, 20.01.2020, 26.04.2020, 20.07.2020, 05.03.2021
# FOSHKplugin@phantasoft.de - http://foshkplugin.phantasoft.de

use File::HomeDir;
use CGI qw/:standard/;
use Cwd 'abs_path';
use IO::Socket::INET;
use HTML::Entities;
use String::Escape qw( unquotemeta );
use LoxBerry::System;
use JSON;
use LWP::Simple qw/get/;
use Config::Simple;
use warnings;
use strict;
no strict "refs"; # we need it for template system
use CGI::Carp qw/fatalsToBrowser/;

my  $myDebug = 0; # set to 1 to enable debugging-messages
my  $prgname = "FOSHKplugin";
my  $prgver = LoxBerry::System::pluginversion();
my  $pluginname = LoxBerry::System::plugindata()->{PLUGINDB_TITLE};
my  $maxdev = 10;
my  $home = File::HomeDir->my_home;
our $lang;
my  $installfolder;
my  $cfg;
my  $conf;
our $template_title;
our $namef;
our $value;
our %query;
our $phrase;
our $phraseplugin;
our $languagefile;
our $languagefileplugin;
our $cache;
our $savedata;
our $MSselectlist;
our $miniserver;
our $fwdtypelist;

our $enabled;
our $helptext;
our $helplink;
our $restartMsg;
our $logdir;
our $linkvorlage = "";

# hier globalen Vars
our $LOX_IP;
our $LOX_PORT;
our $LB_IP;
our $LBU_PORT;
our $LBH_PORT;
our $LBS_PORT;
our $LOX_TIME;
our $USE_METRIC;
our $IGNORE_EMPTY;
our $logfile;
our $rawfile;
our $sndfile;
our $WS_IP;
our $WS_PORT;
our $WS_INTERVAL;
our $FWD_URL;
our $FWD_IGNORE;
our $FWD_INTERVAL;
our $FWD_TYPE;
our $FWD_SID;
our $FWD_PWD;
our $FWD_STATUS;
our $FWD_ENABLE;
our $CSV_NAME;
our $CSV_INTERVAL;
our $CSV_FIELDS;
our $EVAL_VALUES;
our $UDP_MINMAX;
our $UDP_STATRESEND;
our $ADD_ITEMS;
our $FOSHKrunning;

sub clearString($) {
  my $string = shift;
  $string =~ s/#[^|]*//g if defined($string);
  $string =~ s/\s+$// if defined($string);
  $string = "" if not defined $string;
  return $string;
}

sub logPrint {
  my $outstr = $_[0];
  my $timestamp = localtime(time);
  if ($outstr ne "") {
    print STDERR "$timestamp $prgname: $outstr\n";
  } else {
    print STDERR "\n";
  }
}

sub debugPrint {
  my $str = $_[0];
  chomp($str);
  if ($myDebug) { logPrint($str); }
}

# start the cgi-script
debugPrint("cgi start");

# ---------------------------------------
# Read Settings
# ---------------------------------------
debugPrint("read settings start");
my $gencfg = "$home/config/system/general.cfg";
$cfg = new Config::Simple($gencfg);
eval { $installfolder = $cfg->param("BASE.INSTALLFOLDER") };
if ($@) { die "error while getting installfolder from $gencfg" }
eval { $lang = $cfg->param("BASE.LANG") };
if ($@) { die "error while getting lang from $gencfg" }

my $LBPDATA = $ENV{'LBPDATA'};
my $LBPCONFIG = $ENV{'LBPCONFIG'};
my $LBPBIN = $ENV{'LBPBIN'};
$logdir = $ENV{'LBPLOG'}."/$lbpplugindir/";
my $lbip = LoxBerry::System::get_localip();

our $Yes = "Yes";
our $No = "No";
if ($lang eq "de") {
  $Yes = "Ja";
  $No = "Nein";
} 
elsif ($lang eq "nl") {
  $Yes = "Ja";
  $No = "Nee";
}
elsif ($lang eq "sk") {
  $Yes = "Áno";
  $No = "Nie";
}

debugPrint("  prgver: ".$prgver);
debugPrint("  home: ".$home);
debugPrint("  genCFG: ".$gencfg);
debugPrint("  installfolder: ".$installfolder);
debugPrint("  lang: ".$lang);
debugPrint("  LBPDATA: ".$LBPDATA);
debugPrint("  LBPCONFIG: ".$LBPCONFIG);
debugPrint("  LBPBIN: ".$LBPBIN);
debugPrint("  logdir: ".$logdir);
debugPrint("  lbip: ".$lbip);
debugPrint("read settings done");

print "Content-Type: text/html\n\n";

# ---------------------------------------
# Parse URL
# ---------------------------------------
debugPrint("parse url start");
foreach (split(/&/,$ENV{"QUERY_STRING"}))
{
  ($namef,$value) = split(/=/,$_,2);
  $namef =~ tr/+/ /;
  $namef =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
  $value =~ tr/+/ /;
  $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
  $query{$namef} = $value;
}
debugPrint("parse url done");

# ---------------------------------------
# Set parameters coming in - GET over POST
# ---------------------------------------
# alle Parameter einlesen
debugPrint("parse parameters coming in start");
my $p;
my @param_names = param();
foreach $p (@param_names) {
  $value = param($p);
  my $str = substr $p,0,2;
}

if ( param('doNothing') )   {
  #Sicherheitsabfrage abgebrochen; weiter geht's
}

if ( !$query{'LOX_IP'} )   { if ( param('LOX_IP')  ) { $LOX_IP = quotemeta(param('LOX_IP'));         }
else { $LOX_IP = $LOX_IP;  } } else { $LOX_IP = quotemeta($query{'LOX_IP'});   }

if ( param('miniserver') )   {
  $LOX_IP = quotemeta(param('miniserver'));
}

if ( !$query{'LOX_PORT'} )   { if ( param('LOX_PORT')  ) { $LOX_PORT = quotemeta(param('LOX_PORT'));         }
else { $LOX_PORT = $LOX_PORT;  } } else { $LOX_PORT = quotemeta($query{'LOX_PORT'});   }

if ( !$query{'LB_IP'} )   { if ( param('LB_IP')  ) { $LB_IP = quotemeta(param('LB_IP'));         }
else { $LB_IP = $LB_IP;  } } else { $LB_IP = quotemeta($query{'LB_IP'});   }

if ( !$query{'LBU_PORT'} )   { if ( param('LBU_PORT')  ) { $LBU_PORT = quotemeta(param('LBU_PORT'));         }
else { $LBU_PORT = $LBU_PORT;  } } else { $LBU_PORT = quotemeta($query{'LBU_PORT'});   }

if ( !$query{'LBH_PORT'} )   { if ( param('LBH_PORT')  ) { $LBH_PORT = quotemeta(param('LBH_PORT'));         }
else { $LBH_PORT = $LBH_PORT;  } } else { $LBH_PORT = quotemeta($query{'LBH_PORT'});   }

if ( !$query{'LOX_TIME'} )   { if ( param('LOX_TIME')  ) { $LOX_TIME = quotemeta(param('LOX_TIME'));         }
else { $LOX_TIME = $LOX_TIME;  } } else { $LOX_TIME = quotemeta($query{'LOX_TIME'});   }

if ( !$query{'logfile'} )   { if ( param('logfile')  ) { $logfile = quotemeta(param('logfile'));         }
else { $logfile = $logfile;  } } else { $logfile = quotemeta($query{'logfile'});   }

if ( !$query{'rawfile'} )   { if ( param('rawfile')  ) { $rawfile = quotemeta(param('rawfile'));         }
else { $rawfile = $rawfile;  } } else { $rawfile = quotemeta($query{'rawfile'});   }

if ( !$query{'sndfile'} )   { if ( param('sndfile')  ) { $sndfile = quotemeta(param('sndfile'));         }
else { $sndfile = $sndfile;  } } else { $sndfile = quotemeta($query{'sndfile'});   }

if ( !$query{'enabled'} )   { if ( param('enabled')  ) { $enabled = quotemeta(param('enabled'));         }
else { $enabled = $enabled;  } } else { $enabled = quotemeta($query{'enabled'});   }

if ( !$query{'USE_METRIC'} )   { if ( param('USE_METRIC')  ) { $USE_METRIC = quotemeta(param('USE_METRIC'));         }
else { $USE_METRIC = $USE_METRIC;  } } else { $USE_METRIC = quotemeta($query{'USE_METRIC'});   }

if ( !$query{'IGNORE_EMPTY'} )   { if ( param('IGNORE_EMPTY')  ) { $IGNORE_EMPTY = quotemeta(param('IGNORE_EMPTY'));         }
else { $IGNORE_EMPTY = $IGNORE_EMPTY;  } } else { $IGNORE_EMPTY = quotemeta($query{'IGNORE_EMPTY'});   }

if ( !$query{'WS_IP'} )   { if ( param('WS_IP')  ) { $WS_IP = quotemeta(param('WS_IP'));         }
else { $WS_IP = $WS_IP;  } } else { $WS_IP = quotemeta($query{'WS_IP'});   }

if ( !$query{'WS_PORT'} )   { if ( param('WS_PORT')  ) { $WS_PORT = quotemeta(param('WS_PORT'));         }
else { $WS_PORT = $WS_PORT;  } } else { $WS_PORT = quotemeta($query{'WS_PORT'});   }

if ( !$query{'WS_INTERVAL'} )   { if ( param('WS_INTERVAL')  ) { $WS_INTERVAL = quotemeta(param('WS_INTERVAL'));         }
else { $WS_INTERVAL = $WS_INTERVAL;  } } else { $WS_INTERVAL = quotemeta($query{'WS_INTERVAL'});   }

if ( !$query{'FWD_URL'} )   { if ( param('FWD_URL')  ) { $FWD_URL = quotemeta(param('FWD_URL'));         }
else { $FWD_URL = $FWD_URL;  } } else { $FWD_URL = quotemeta($query{'FWD_URL'});   }

if ( !$query{'FWD_TYPE'} )   { if ( param('FWD_TYPE')  ) { $FWD_TYPE = quotemeta(param('FWD_TYPE'));         }
else { $FWD_TYPE = $FWD_TYPE;  } } else { $FWD_TYPE = quotemeta($query{'FWD_TYPE'});   }

if ( !$query{'FWD_SID'} )   { if ( param('FWD_SID')  ) { $FWD_SID = quotemeta(param('FWD_SID'));         }
else { $FWD_SID = $FWD_SID;  } } else { $FWD_SID = quotemeta($query{'FWD_SID'});   }

if ( !$query{'FWD_PWD'} )   { if ( param('FWD_PWD')  ) { $FWD_PWD = quotemeta(param('FWD_PWD'));         }
else { $FWD_PWD = $FWD_PWD;  } } else { $FWD_PWD = quotemeta($query{'FWD_PWD'});   }

if ( !$query{'FWD_STATUS'} )   { if ( param('FWD_STATUS')  ) { $FWD_STATUS = quotemeta(param('FWD_STATUS'));         }
else { $FWD_STATUS = $FWD_STATUS;  } } else { $FWD_STATUS = quotemeta($query{'FWD_STATUS'});   }

if ( !$query{'FWD_ENABLE'} )   { if ( param('FWD_ENABLE')  ) { $FWD_ENABLE = quotemeta(param('FWD_ENABLE'));         }
else { $FWD_ENABLE = $FWD_ENABLE;  } } else { $FWD_ENABLE = quotemeta($query{'FWD_ENABLE'});   }

if ( !$query{'FWD_IGNORE'} )   { if ( param('FWD_IGNORE')  ) { $FWD_IGNORE = quotemeta(param('FWD_IGNORE'));         }
else { $FWD_IGNORE = $FWD_IGNORE;  } } else { $FWD_IGNORE = quotemeta($query{'FWD_IGNORE'});   }

if ( !$query{'FWD_INTERVAL'} )   { if ( param('FWD_INTERVAL')  ) { $FWD_INTERVAL = quotemeta(param('FWD_INTERVAL'));         }
else { $FWD_INTERVAL = $FWD_INTERVAL;  } } else { $FWD_INTERVAL = quotemeta($query{'FWD_INTERVAL'});   }

if ( !$query{'CSV_NAME'} )   { if ( param('CSV_NAME')  ) { $CSV_NAME = quotemeta(param('CSV_NAME'));         }
else { $CSV_NAME = $CSV_NAME;  } } else { $CSV_NAME = quotemeta($query{'CSV_NAME'});   }

if ( !$query{'CSV_INTERVAL'} )   { if ( param('CSV_INTERVAL')  ) { $CSV_INTERVAL = quotemeta(param('CSV_INTERVAL'));         }
else { $CSV_INTERVAL = $CSV_INTERVAL;  } } else { $CSV_INTERVAL = quotemeta($query{'CSV_INTERVAL'});   }

if ( !$query{'CSV_FIELDS'} )   { if ( param('CSV_FIELDS')  ) { $CSV_FIELDS = quotemeta(param('CSV_FIELDS'));         }
else { $CSV_FIELDS = $CSV_FIELDS;  } } else { $CSV_FIELDS = quotemeta($query{'CSV_FIELDS'});   }

if ( !$query{'LBS_PORT'} )   { if ( param('LBS_PORT')  ) { $LBS_PORT = quotemeta(param('LBS_PORT'));         }
else { $LBS_PORT = $LBS_PORT;  } } else { $LBS_PORT = quotemeta($query{'LBS_PORT'});   }

if ( !$query{'EVAL_VALUES'} )   { if ( param('EVAL_VALUES')  ) { $EVAL_VALUES = quotemeta(param('EVAL_VALUES'));         }
else { $EVAL_VALUES = $EVAL_VALUES;  } } else { $EVAL_VALUES = quotemeta($query{'EVAL_VALUES'});   }

if ( !$query{'UDP_MINMAX'} )   { if ( param('UDP_MINMAX')  ) { $UDP_MINMAX = quotemeta(param('UDP_MINMAX'));         }
else { $UDP_MINMAX = $UDP_MINMAX;  } } else { $UDP_MINMAX = quotemeta($query{'UDP_MINMAX'});   }

if ( !$query{'UDP_STATRESEND'} )   { if ( param('UDP_STATRESEND')  ) { $UDP_STATRESEND = quotemeta(param('UDP_STATRESEND'));         }
else { $UDP_STATRESEND = $UDP_STATRESEND;  } } else { $UDP_STATRESEND = quotemeta($query{'UDP_STATRESEND'});   }

if ( !$query{'ADD_ITEMS'} )   { if ( param('ADD_ITEMS')  ) { $ADD_ITEMS = quotemeta(param('ADD_ITEMS'));         }
else { $ADD_ITEMS = $ADD_ITEMS;  } } else { $ADD_ITEMS = quotemeta($query{'ADD_ITEMS'});   }

if ( param('getWSIP') )   {
  $WS_IP = trim(qx(python3 $LBPDATA/$lbpplugindir/foshkplugin.py -getWSIP));
  #logPrint("get weather station addr: ".$WS_IP);
}

if ( param('getWSPORT') )   {
  $WS_PORT = trim(qx(python3 $LBPDATA/$lbpplugindir/foshkplugin.py -getWSPORT));
  #logPrint("get weather station port: ".$WS_PORT);
}

if ( param('getWSINTERVAL') )   {
  $WS_INTERVAL = trim(qx(python3 $LBPDATA/$lbpplugindir/foshkplugin.py -getWSINTERVAL));
  #logPrint("get weather station interval: ".$WS_PORT);
}

if ( param('getCSVHEADER') )   {
  $CSV_FIELDS = trim(qx(python3 $LBPDATA/$lbpplugindir/foshkplugin.py -getCSVHEADER));
}

if ( param('getLBIP') )   {
  $LB_IP = $lbip;
}

if ( param('restartPlugin') )   {
  logPrint("restarting service ".$prgname." v".$prgver);
  qx($LBPBIN/$lbpplugindir/service.sh restart);
}

if ( param('doVorlage') )   {
  $LOX_IP = unquotemeta($LOX_IP);
  # Speicherort fuer die Datei muss downloadbar sein
  my $vdatei = $pluginname."-in-out.xml";
  open(my $fh, ">", "$vdatei") || die "can not write to $vdatei";
  #print $fh "$lbip $LBU_PORT ".unquotemeta($LOX_IP)." $LOX_PORT\n";
  my $line = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
  $line .= "<VirtualInUdp Title=\"".$pluginname."\" Comment=\"\" Address=\"\" Port=\"$LOX_PORT\">\n";
  print $fh $line;
  # hier die Input-Ports
  my $mSID = "FOSHKweather";
  my @sys_cmd = ("running","wswarning","stormwarning","tswarning","sensorwarning","batterywarning","updatewarning","leakwarning","co2warning");
  foreach my $cmd ( @sys_cmd ) {
    print $fh "\t<VirtualInUdpCmd Title=\"FOSHK-$cmd\" Comment=\"FOSHK-$cmd\" Address=\"\" Check=\"SID=$mSID\\i$cmd=\\i\\v\" Signed=\"false\" Analog=\"true\" SourceValLow=\"0\" DestValLow=\"0\" SourceValHigh=\"100\" DestValHigh=\"100\" DefVal=\"0\" MinVal=\"-10000\" MaxVal=\"10000\"/>\n";
  }
  my $i;
  my $cmd;
  my $cmdC;
  my $BFcmd;
  @sys_cmd = (
              "AqPM2.5","baromabshpa","baromhpa","baromrelhpa","dailyrainmm","dewptc","eventrainmm","feelslikec","heatindexc","hourlyrainmm","humidity","humidityin",
              "lightning","lightning_num","lightning_time","lightning_loxtime","loxtime","maxdailygust","monthlyrainmm","rainratemm","solarradiation","tempc","tempinc",
              "totalrainmm","uv","weeklyrainmm","wh26batt","wh40batt","wh57batt","wh65batt","wh68batt","wh80batt","co2_batt","windchillc","winddir","windgustkmh",
              "windspeedkmh","yearlyrainmm","ptrend1","pchange1","ptrend3","pchange3","wnowlvl","wproglvl","tc_co2","humidity_co2","pm25_co2","pm25_24h_co2","pm10_co2",
              "pm10_24h_co2","co2","co2_24h","co2lvl","pm25_AQI_co2","pm25_AQIlvl_co2","pm25_AQI_24h_co2","pm25_AQIlvl_24h_co2","pm10_AQI_co2","pm10_AQIlvl_co2",
              "pm10_AQI_24h_co2","pm10_AQIlvl_24h_co2","windspdkmh_avg10m","winddir_avg10m","windgustkmh_max10m","brightness","cloudbase","sunhours",
              # v0.08 min/max values
              "tempc_min","tempc_min_time","tempc_max","tempc_max_time","windchillc_min","windchillc_min_time","windchillc_max","windchillc_max_time",
              "heatindexc_min","heatindexc_min_time","heatindexc_max","heatindexc_max_time","tempinc_min","tempinc_min_time","tempinc_max","tempinc_max_time",
              "baromrelhpa_min","baromrelhpa_min_time","baromrelhpa_max","baromrelhpa_max_time","feelslikec_min","feelslikec_min_time","feelslikec_max","feelslikec_max_time",
              "dewptc_min","dewptc_min_time","dewptc_max","dewptc_max_time","humidity_min","humidity_min_time","humidity_max","humidity_max_time",
              "windspeedkmh_max","windspeedkmh_max_time","windgustkmh_max","windgustkmh_max_time",
              # v0.09 additional keys
              "runtime","ws1900batt","rrain_piezo","erain_piezo","hrain_piezo","drain_piezo","wrain_piezo","mrain_piezo","yrain_piezo","wh90batt","ws90cap_volt",
              "ws90_ver","gain10_piezo","gain20_piezo","gain30_piezo","gain40_piezo","gain50_piezo","rfdata"
              );
  foreach $cmd ( @sys_cmd ) {
    # v0.09: Befehlserkennung anpassen: "co2=" ist mehrfach vorhanden, kann daher nicht eindeutig identifiziert werden
    if ($cmd eq "co2") { $BFcmd = " $cmd"; } else { $BFcmd = $cmd; }
    print $fh "\t<VirtualInUdpCmd Title=\"FOSHK-$cmd\" Comment=\"FOSHK-$cmd\" Address=\"\" Check=\"SID=$mSID\\i$BFcmd=\\i\\v\" Signed=\"false\" Analog=\"true\" SourceValLow=\"0\" DestValLow=\"0\" SourceValHigh=\"100\" DestValHigh=\"100\" DefVal=\"0\" MinVal=\"-10000\" MaxVal=\"10000\"/>\n";
  }
  # Text-sensors 2do
  #@sys_cmd = ("wprogtxt","wnowtxt");
  #foreach $cmd ( @sys_cmd ) {
  #  print $fh "\t<VirtualInUdpCmd Title=\"FOSHK-$cmd\" Comment=\"FOSHK-$cmd\" Address=\"\" Check=\"SID=$mSID\\i$cmd=\\i\\v\" Signed=\"false\" Analog=\"true\" SourceValLow=\"0\" DestValLow=\"0\" SourceValHigh=\"100\" DestValHigh=\"100\" DefVal=\"0\" MinVal=\"-10000\" MaxVal=\"10000\"/>\n";
  #}
  # 4-channel-sensors
  @sys_cmd = ("pm25_avg_24h_ch§","pm25_ch§","pm25batt§","leak_ch§","leakbatt§","pm25_AQI_ch§","pm25_AQI_avg_24h_ch§","pm25_AQIlvl_ch§","pm25_AQIlvl_avg_24h_ch§","pm10_AQI_ch§","pm10_AQI_avg_24h_ch§","pm10_AQIlvl_ch§","pm10_AQIlvl_avg_24h_ch§");
  for $i (1..4) {
    foreach $cmdC ( @sys_cmd ) {
      $cmd = $cmdC;
      $cmd =~ s/§/$i/g;
      print $fh "\t<VirtualInUdpCmd Title=\"FOSHK-$cmd\" Comment=\"FOSHK-$cmd\" Address=\"\" Check=\"SID=$mSID\\i$cmd=\\i\\v\" Signed=\"false\" Analog=\"true\" SourceValLow=\"0\" DestValLow=\"0\" SourceValHigh=\"100\" DestValHigh=\"100\" DefVal=\"0\" MinVal=\"-10000\" MaxVal=\"10000\"/>\n";
    }
  }
  # 8-channel-sensors
  @sys_cmd = ("batt§","humidity§","soilbatt§","soilmoisture§","temp§c","tf_ch§c","tf_batt§","leafwetness_ch§","leaf_batt§");
  for $i (1..8) {
    foreach $cmdC ( @sys_cmd ) {
      $cmd = $cmdC;
      $cmd =~ s/§/$i/ig;
      print $fh "\t<VirtualInUdpCmd Title=\"FOSHK-$cmd\" Comment=\"FOSHK-$cmd\" Address=\"\" Check=\"SID=$mSID\\i$cmd=\\i\\v\" Signed=\"false\" Analog=\"true\" SourceValLow=\"0\" DestValLow=\"0\" SourceValHigh=\"100\" DestValHigh=\"100\" DefVal=\"0\" MinVal=\"-10000\" MaxVal=\"10000\"/>\n";
    }
  }
  # virtual inputs for W4L
  @sys_cmd = (
               #"title","comment-de","comment-en","cmd"
               ["w4l_cur_hu","Rel. Luftfeuchtigkeit","Rel. Humidity","humidity"],
               ["w4l_cur_pr","Luftdruck","Pressure","baromrelhpa"],
               ["w4l_cur_prec_1hr","Niederschlag 1 Std.","Precipitation 1hr","hourlyrainmm"],
               ["w4l_cur_prec_today","Niederschlag heute","Precipitation Today","dailyrainmm"],
               ["w4l_cur_sr","Solareinstrahlung","Solar Radiation","solarradiation"],
               ["w4l_cur_tt","Temperatur","Temperature","tempc"],
               ["w4l_cur_uvi","UV Index","UV Index","uv"],
               ["w4l_cur_w_dir","Windrichtung Grad","Wind Dir Degrees","winddir"],
               ["w4l_cur_w_gu","Windböen","Wind Gust","windgustkmh"],
               ["w4l_cur_w_sp","Windgeschwindigkeit","Wind Speed","windspeedkmh"],
               ["w4l_cur_w_ch","Windchill","Windchill","windchillc"],
               ["w4l_cur_tt_fl","gefühlte Temperatur","Feelslike Temp","feelslikec"],
               ["w4l_cur_dp","Taupunkt","Dew Point","dewpointc"],
               ["w4l_cur_hi","Hitzeindex","Heat Index","heatindexc"]
             );
  my $lptr;
  if ($lang eq "de") {
    $lptr = 1;
  } else {
    $lptr = 2;
  }
  foreach $cmd ( @sys_cmd ) {
    print $fh "\t<VirtualInUdpCmd Title=\"${$cmd}[0]\" Comment=\"${$cmd}[$lptr]\" Address=\"\" Check=\"SID=$mSID\\i${$cmd}[3]=\\i\\v\" Signed=\"false\" Analog=\"true\" SourceValLow=\"0\" DestValLow=\"0\" SourceValHigh=\"100\" DestValHigh=\"100\" DefVal=\"0\" MinVal=\"-10000\" MaxVal=\"10000\"/>\n";
  }

  $line = "</VirtualInUdp>\n\n";
  # hier der Block fuer die virtuellen Outs
  $line .= "<VirtualOut Title=\"".$pluginname."\" Comment=\"\" Address=\"/dev/udp/$lbip/$LBU_PORT\" CmdInit=\"\" CloseAfterSend=\"true\" CmdSep=\" \">\n";
  print $fh $line;
  # hier die Output-Ports: Titel,comment,start,stop,Analog
  @sys_cmd = (
               ["FOSHK-Reboot","FOSHK-Reboot","SID=FOSHKplugin,System.reboot","","false"],
               ["FOSHK-Shutdown","FOSHK-Shutdown","SID=FOSHKplugin,Plugin.shutdown","","false"],
               ["FOSHK-getStatus","FOSHK-getStatus","SID=FOSHKplugin,Plugin.getstatus","","false"],
               ["FOSHK-debugOn","FOSHK-debugOn","SID=FOSHKplugin,Plugin.debug=enable","","false"],
               ["FOSHK-debugOff","FOSHK-debugOff","SID=FOSHKplugin,Plugin.debug=disable","","false"],
               ["FOSHK-leakWarnOn","FOSHK-LeakWarnOn","SID=FOSHKplugin,Plugin.leakwarning=enable","","false"],
               ["FOSHK-leakWarnOff","FOSHK-LeakWarnOff","SID=FOSHKplugin,Plugin.leakwarning=disable","","false"],
               ["FOSHK-co2WarnOn","FOSHK-co2WarnOn","SID=FOSHKplugin,Plugin.co2warning=enable","","false"],
               ["FOSHK-co2WarnOff","FOSHK-co2WarnOff","SID=FOSHKplugin,Plugin.co2warning=disable","","false"],
               ["FOSHK-PushoverOn","FOSHK-PushoverOn","SID=FOSHKplugin,Plugin.pushover=enable","","false"],
               ["FOSHK-PushoverOff","FOSHK-PushoverOff","SID=FOSHKplugin,Plugin.pushover=disable","","false"],
               ["FOSHK-intvlwarningOn","FOSHK-intvlwarningOn","SID=FOSHKplugin,Plugin.intvlwarning=enable","","false"],
               ["FOSHK-intvlwarningOff","FOSHK-intvlwarningOff","SID=FOSHKplugin,Plugin.intvlwarning=disable","","false"]
             );
  foreach my $ocmd ( @sys_cmd ) {
    print $fh "\t<VirtualOutCmd Title=\"${$ocmd}[0]\" Comment=\"${$ocmd}[1]\" CmdOnMethod=\"GET\" CmdOffMethod=\"GET\" CmdOn=\"${$ocmd}[2]\" CmdOnHTTP=\"\" CmdOnPost=\"\" CmdOff=\"${$ocmd}[3]\" CmdOffHTTP=\"\" CmdOffPost=\"\" Analog=\"${$ocmd}[4]\" Repeat=\"0\" RepeatRate=\"0\"/>\n";
  }
  $line = "</VirtualOut>\n";
  print $fh $line;
  close($fh);
  my $filesize = -s "$vdatei";
  # jetzt irgendwie einen Download initiieren
  $linkvorlage ="<a id=\"autoload\" href=\"$vdatei\" download=\"$vdatei\" type=\"application/octet-stream\"></a>";
}

if ( param('setWSconfig') )   {
  # setWSconfig(ws_ip, ws_port, custom_host, custom_port, custom_interval)
  my $def_LB_IP;
  if (! defined $LB_IP || $LB_IP eq "") { $def_LB_IP = $lbip; } else { $def_LB_IP = $LB_IP; }
  my $result = qx(python3 $LBPDATA/$lbpplugindir/foshkplugin.py -setWSconfig $WS_IP $WS_PORT $def_LB_IP $LBH_PORT $WS_INTERVAL);
  # im Fehlerfall eine Warnung auf den Schirm bringen:
  if ($result =~ /<ERROR>/i) {
    logPrint($result." - please try with WS View!");
    print "<script language=\"JavaScript\">alert('$result - please try with WS View!');</script>";
  } else {
    #logPrint($result)
  }
}

if ( param('CheckLBUPort') )   {
  my $udpok = "";
  $udpok = trim(qx(python3 $LBPDATA/$lbpplugindir/foshkplugin.py -checkLBUPort $LBU_PORT));
  if ($udpok eq "ok") {
    $LBU_PORT = $LBU_PORT;
  } else {
    $LBU_PORT = 0;
  }
  #logPrint("check UDP Port ".$LBU_PORT." if usable: ".$udpok);
}

if ( param('CheckLBHPort') )   {
  my $udpok = "";
  $udpok = trim(qx(python3 $LBPDATA/$lbpplugindir/foshkplugin.py -checkLBHPort $LBH_PORT));
  if ($udpok eq "ok") {
    $LBH_PORT = $LBH_PORT;
  } else {
    $LBH_PORT = 0;
  }
  #logPrint("check HTTP-Port ".$LBH_PORT." if usable: ".$udpok);
}

if ( param('CheckLBSPort') )   {
  my $udpok = "";
  $udpok = trim(qx(python3 $LBPDATA/$lbpplugindir/foshkplugin.py -checkLBHPort $LBS_PORT));
  if ($udpok eq "ok") {
    $LBS_PORT = $LBS_PORT;
  } else {
    $LBS_PORT = 0;
  }
  #logPrint("check HTTPS-Port ".$LBS_PORT." if usable: ".$udpok);
}

if (defined $LOX_IP) { debugPrint("  LOX_IP: ".$LOX_IP); }
if (defined $LOX_PORT) { debugPrint("  LOX_PORT: ".$LOX_PORT); }
if (defined $LB_IP) { debugPrint("  LB_IP: ".$LB_IP); }
if (defined $LBU_PORT) { debugPrint("  LBU_PORT: ".$LBU_PORT); }
if (defined $LBH_PORT) { debugPrint("  LBH_PORT: ".$LBH_PORT); }
if (defined $LOX_TIME) { debugPrint("  LOX_TIME: ".$LOX_TIME); }
if (defined $LBU_PORT) { debugPrint("  LBU_PORT: ".$LBU_PORT); }
if (defined $logfile) { debugPrint("  logfile: ".$logfile); }
if (defined $rawfile) { debugPrint("  rawfile: ".$rawfile); }
if (defined $sndfile) { debugPrint("  sndfile: ".$sndfile); }
if (defined $enabled) { debugPrint("  enabled: ".$enabled); }
if (defined $USE_METRIC) { debugPrint("  USE_METRIC: ".$USE_METRIC); }
if (defined $IGNORE_EMPTY) { debugPrint("  IGNORE_EMPTY: ".$IGNORE_EMPTY); }
if (defined $WS_IP) { debugPrint("  WS_IP: ".$WS_IP); }
if (defined $WS_PORT) { debugPrint("  WS_PORT: ".$WS_PORT); }
if (defined $WS_INTERVAL) { debugPrint("  WS_INTERVAL: ".$WS_INTERVAL); }
if (defined $FWD_URL) { debugPrint("  FWD_URL: ".$FWD_URL); }
if (defined $FWD_TYPE) { debugPrint("  FWD_TYPE: ".$FWD_TYPE); }
if (defined $FWD_SID) { debugPrint("  FWD_SID: ".$FWD_SID); }
if (defined $FWD_PWD) { debugPrint("  FWD_PWD: ".$FWD_PWD); }
if (defined $FWD_STATUS) { debugPrint("  FWD_STATUS: ".$FWD_STATUS); }
if (defined $FWD_ENABLE) { debugPrint("  FWD_ENABLE: ".$FWD_ENABLE); }
if (defined $FWD_IGNORE) { debugPrint("  FWD_IGNORE: ".$FWD_IGNORE); }
if (defined $FWD_INTERVAL) { debugPrint("  FWD_INTERVAL: ".$FWD_INTERVAL); }
if (defined $CSV_NAME) { debugPrint("  CSV_NAME: ".$CSV_NAME); }
if (defined $CSV_INTERVAL) { debugPrint("  CSV_INTERVAL: ".$CSV_INTERVAL); }
if (defined $CSV_FIELDS) { debugPrint("  CSV_FIELDS: ".$CSV_FIELDS); }
if (defined $LBS_PORT) { debugPrint("  LBS_PORT: ".$LBS_PORT); }
if (defined $EVAL_VALUES) { debugPrint("  EVAL_VALUES: ".$EVAL_VALUES); }
if (defined $UDP_MINMAX) { debugPrint("  UDP_MINMAX: ".$UDP_MINMAX); }
if (defined $UDP_STATRESEND) { debugPrint("  UDP_STATRESEND: ".$UDP_STATRESEND); }
if (defined $ADD_ITEMS) { debugPrint("  ADD_ITEMS: ".$ADD_ITEMS); }
debugPrint("parse parameters coming in done");

$miniserver = "";
if ($lang eq "de") {
  $helptext = "<b>grundsätzlicher Ablauf:</b><br/><br/>1. hier die erforderlichen Daten eingeben<br/><br/>2. Settings abspeichern mit <strong>Speichern</strong><br/><br/>3. über <strong>WS-Set</strong> die Konfigurationsdaten zur Wetterstation übertragen<br/><br/>4. mit <strong>Restart</strong> das Plugin mit den neuen Settings neustarten<br/><br/>5. optional per <strong>Vorlage</strong> die Vorlagen-Datei für Loxone downloaden<br/><br/><br/><b>Speichern</b><br/>speichert die Config-Datei<br/><br/><b>WS-Set</b><br/>Schreiben der Wetterstation-Konfiguration<br/><br/><b>Restart</b><br/>Neustart des FOSHKplugin mit den aktualisierten Settings<br/><br/><b>Vorlage</b><br/>Download der Loxone-Vorlagen-Datei<br/><br/>";
} elsif ($lang eq "nl") {
  $helptext = "<b>Basisprocedure:</b><br/><br/>1. voer hier de vereiste gegevens in <br/><br/>2. Sla instellingen op met <strong>Opslaan</strong><br/><br/>3. gebruik <strong>WS-Set</strong> om de instellingen van uw weerstation te lezen / bewerken.<br/><br/>4. met <strong>Herstarten</strong> herstart de plugin met de nieuwe instellingen<br/><br/>5. optionele <strong>Template</strong> download het templatebestand voor Loxone<br/><br/><b>Opslaan</b><br/> slaat het configuratiebestand op<br/><br/><b>WS-Set</b><br/>Bewerk de configuratie van uw weerstation<br/><br/><b>Herstarten</b><br/>Start de FOSHKplugin opnieuw met de bijgewerkte instellingen<br/><br/><b>Template</b><br/>Download het Loxone-templatebestand<br/><br/>";
} elsif ($lang eq "sk") {
  $helptext = "<b>základný postup:</b><br/><br/>1. sem zadajte požadované údaje<br/><br/>2. Uložte nastavenia pomocou <strong>Uložiť</strong><br/><br/>3. Preneste konfiguračné údaje na meteorologickú stanicu pomocou <strong>WS-Set</strong>.<br/><br/>4. pomocou <strong>Reštartovať</strong> reštartujte doplnok s novými nastaveniami.<br/><br/>5. voliteľne stiahnite súbor šablóny pre Loxone pomocou <strong>Šablóna</strong><br/><br/><br/><b>Uložiť</b><br/>uloží konfiguračný súbor<br/><br/><b>WS-Set</b><br/>Zápis konfigurácie meteorologickej stanice<br/><br/><b>Reštartovať</b><br/>Reštartovať FOSHKplugin s aktualizovanými nastaveniami<br/><br/><b>Šablóna</b><br/>Stiahnite si súbor so šablónou Loxone<br/><br/>";
} else {
  $helptext = "<b>Basic procedure:</b><br/><br/>1. enter the required data here<br/><br/>2. Save settings with <strong>Save</strong><br/><br/>3. use <strong>WS-Set</strong> to read/edit the settings of your weather station.<br/><br/>4. with <strong>Restart</strong> restart the plugin with the new settings<br/><br/>5. (optional) download the template file for Loxone with <strong>Template</strong><br/><br/><b>Save</b><br/>saves the config file<br/><br/><b>WS-Set</b><br/>Save the configuration of your weatherstation<br/><br/><b>Restart</b><br/>Restart the FOSHKplugin with the updated settings<br/><br/><b>Template</b><br/>Download the Loxone template file<br/><br/>";
}
$helplink = "http://foshkplugin.phantasoft.de";
$restartMsg = "";

# ---------------------------------------
# Save settings to config file
# ---------------------------------------
debugPrint("save settings start");
if (param('savedata')) {
  #print STDERR "n. savedata in index\n";
  my $cfgfile = "$home/config/plugins/$lbpplugindir/foshkplugin.conf";
  debugPrint("  cfgfile: $cfgfile");
  $conf = new Config::Simple($cfgfile);
  # v0.08: fix string wrongly recognized as an array
  my $line = "";
  foreach $line ($conf->param()) {
    if (index($line,"FWD_PWD")) {
      if (ref($conf->param(${line})) eq 'ARRAY') { $conf->param($line, join(",",@{$conf->param(${line})})); }
    }
  }
  $conf->param('Config.LOX_IP', unquotemeta($LOX_IP));
  $conf->param('Config.LOX_PORT', unquotemeta($LOX_PORT));
  $conf->param('Config.LB_IP', unquotemeta($LB_IP));
  $conf->param('Config.LBU_PORT', unquotemeta($LBU_PORT));
  $conf->param('Config.LBH_PORT', unquotemeta($LBH_PORT));
  $conf->param('Config.LOX_TIME', unquotemeta($LOX_TIME));
  $conf->param('Config.ENABLED', unquotemeta($enabled));
  $conf->param('Config.USE_METRIC', unquotemeta($USE_METRIC));
  $conf->param('Config.IGNORE_EMPTY', unquotemeta($IGNORE_EMPTY));
  $conf->param('Logging.logfile', unquotemeta($logfile));
  $conf->param('Logging.rawfile', unquotemeta($rawfile));
  $conf->param('Logging.sndfile', unquotemeta($sndfile));
  $conf->param('Weatherstation.WS_IP', unquotemeta($WS_IP));
  $conf->param('Weatherstation.WS_PORT', unquotemeta($WS_PORT));
  $conf->param('Weatherstation.WS_INTERVAL', unquotemeta($WS_INTERVAL));
  $conf->param('Forward.FWD_URL', unquotemeta($FWD_URL));
  $conf->param('Forward.FWD_IGNORE', "\"".unquotemeta($FWD_IGNORE)."\"");
  $conf->param('Forward.FWD_INTERVAL', unquotemeta($FWD_INTERVAL));
  $conf->param('Forward.FWD_TYPE', unquotemeta($FWD_TYPE));
  $conf->param('Forward.FWD_SID', unquotemeta($FWD_SID));
  $conf->param('Forward.FWD_PWD', unquotemeta($FWD_PWD));
  $conf->param('Forward.FWD_STATUS', unquotemeta($FWD_STATUS));
  $conf->param('Forward.FWD_ENABLE', unquotemeta($FWD_ENABLE));
  $conf->param('CSV.CSV_NAME', unquotemeta($CSV_NAME));
  $conf->param('CSV.CSV_FIELDS', "\"".unquotemeta($CSV_FIELDS)."\"");
  $conf->param('CSV.CSV_INTERVAL', unquotemeta($CSV_INTERVAL));
  $conf->param('Config.LBS_PORT', unquotemeta($LBS_PORT));
  $conf->param('Export.EVAL_VALUES', unquotemeta($EVAL_VALUES));
  $conf->param('Export.UDP_MINMAX', unquotemeta($UDP_MINMAX));
  $conf->param('Config.UDP_STATRESEND', unquotemeta($UDP_STATRESEND));
  $conf->param('Export.ADD_ITEMS', unquotemeta($ADD_ITEMS));
  $conf->save();
  debugPrint("  settings saved to $cfgfile");
}
debugPrint("save settings done");

# ---------------------------------------
# Parse config file
# ---------------------------------------
#print STDERR "4. parse config file\n";
debugPrint("parse config file start");
my $cfgfile = "$home/config/plugins/$lbpplugindir/foshkplugin.conf";
debugPrint("  cfgfile: $cfgfile");
if (-e $cfgfile) {
  #print STDERR "$cfgfile found!\n";
  $conf = new Config::Simple("$cfgfile");
} else {
  #print STDERR "$cfgfile not found!\n";
  # hier den default-dummy kreieren
  $conf = new Config::Simple(syntax=>'ini');
  $conf->param('Config.LBU_PORT',$LBU_PORT);
  $conf->param('Weatherstation.WS_IP',$WS_IP);
  $conf->param('Weatherstation.WS_INTERVAL','60');
  $conf->write($cfgfile);
}

# v0.08: fix string wrongly recognized as an array
my $line = "";
foreach $line ($conf->param()) {
  if (index($line,"FWD_PWD")) {
    if (ref($conf->param(${line})) eq 'ARRAY') { $conf->param($line, join(",",@{$conf->param(${line})})); }
  }
}

#$conf = new Config::Simple("$home/config/plugins/$lbpplugindir/foshkplugin.conf");
#print STDERR "OLI: *$conf*\n";
$LOX_IP = clearString(encode_entities($conf->param('Config.LOX_IP')));
$LOX_PORT = clearString(encode_entities($conf->param('Config.LOX_PORT')));
$LB_IP = clearString(encode_entities($conf->param('Config.LB_IP')));
$LBU_PORT = clearString(encode_entities($conf->param('Config.LBU_PORT')));
$LBH_PORT = clearString(encode_entities($conf->param('Config.LBH_PORT')));
$LOX_TIME = clearString(encode_entities($conf->param('Config.LOX_TIME')));
$logfile = clearString(encode_entities($conf->param('Logging.logfile')));
$rawfile = clearString(encode_entities($conf->param('Logging.rawfile')));
$sndfile = clearString(encode_entities($conf->param('Logging.sndfile')));
$enabled = clearString(encode_entities($conf->param('Config.ENABLED')));
$USE_METRIC = clearString(encode_entities($conf->param('Config.USE_METRIC')));
$IGNORE_EMPTY = clearString(encode_entities($conf->param('Config.IGNORE_EMPTY')));
$WS_IP = clearString(encode_entities($conf->param('Weatherstation.WS_IP')));
$WS_PORT = clearString(encode_entities($conf->param('Weatherstation.WS_PORT')));
$WS_INTERVAL = clearString(encode_entities($conf->param('Weatherstation.WS_INTERVAL')));
$FWD_URL = clearString(encode_entities($conf->param('Forward.FWD_URL')));
$FWD_IGNORE = clearString(encode_entities($conf->param('Forward.FWD_IGNORE')));
$FWD_INTERVAL = clearString(encode_entities($conf->param('Forward.FWD_INTERVAL')));
$FWD_TYPE = clearString(encode_entities($conf->param('Forward.FWD_TYPE')));
if ($FWD_TYPE ne "UDPGET" and $FWD_TYPE ne "RAW" and $FWD_TYPE ne "EW" and $FWD_TYPE ne "RAWEW" and $FWD_TYPE ne "EWRAW" and 
  $FWD_TYPE ne "LD" and $FWD_TYPE ne "CSV" and $FWD_TYPE ne "RAWCSV" and $FWD_TYPE ne "CSVRAW" and $FWD_TYPE ne "RAWUDP" and 
  $FWD_TYPE ne "UDPRAW" and $FWD_TYPE ne "UDP" and $FWD_TYPE ne "AMB" and $FWD_TYPE ne "RAWAMB" and $FWD_TYPE ne "AMBRAW" and 
  $FWD_TYPE ne "WC" and $FWD_TYPE ne "MT" and $FWD_TYPE ne "AWEKAS" and $FWD_TYPE ne "WETTERCOM" and $FWD_TYPE ne "WEATHER365" and 
  $FWD_TYPE ne "REALTIMETXT" and $FWD_TYPE ne "CLIENTRAWTXT" and $FWD_TYPE ne "CSVFILE" and $FWD_TYPE ne "TXTFILE" and 
  $FWD_TYPE ne "TEXTFILE" and $FWD_TYPE ne "MQTTMET" and $FWD_TYPE ne "MQTTIMP" and $FWD_TYPE ne "WSWIN" and $FWD_TYPE ne "INFLUXMET" and 
  $FWD_TYPE ne "INFLUXIMP" and $FWD_TYPE ne "RAWTEXT" and $FWD_TYPE ne "EWUDP" and $FWD_TYPE ne "UDPEW" and $FWD_TYPE ne "APRS") {
  $FWD_TYPE = "WU";
}
$FWD_SID = clearString(encode_entities($conf->param('Forward.FWD_SID')));
$FWD_PWD = clearString(encode_entities($conf->param('Forward.FWD_PWD')));
$FWD_STATUS = clearString(encode_entities($conf->param('Forward.FWD_STATUS')));
$FWD_ENABLE = clearString(encode_entities($conf->param('Forward.FWD_ENABLE')));
$CSV_NAME = clearString(encode_entities($conf->param('CSV.CSV_NAME')));
$CSV_INTERVAL = clearString(encode_entities($conf->param('CSV.CSV_INTERVAL')));
$CSV_FIELDS = clearString(encode_entities($conf->param('CSV.CSV_FIELDS')));
$LBS_PORT = clearString(encode_entities($conf->param('Config.LBS_PORT')));
$EVAL_VALUES = clearString(encode_entities($conf->param('Export.EVAL_VALUES')));
$UDP_MINMAX = clearString(encode_entities($conf->param('Export.UDP_MINMAX')));
$UDP_STATRESEND = clearString(encode_entities($conf->param('Config.UDP_STATRESEND')));
$ADD_ITEMS = clearString(encode_entities($conf->param('Export.ADD_ITEMS')));

#print STDERR "vorher: $FWD_IGNORE --> $FWD_IGNORE\n";
#print STDERR "URL: $FWD_URL\n";

# ---------------------------------------
# Set Enabled / Disabled switch
# ---------------------------------------
if ($enabled eq "True") {
  $enabled = "<option value=\"False\">".$No."</option><option value=\"True\" selected>".$Yes."</option>";
  qx($LBPBIN/$lbpplugindir/service.sh start);
} else {
  $enabled = "<option value=\"False\" selected>".$No."</option><option value=\"True\">".$Yes."</option>";
  qx($LBPBIN/$lbpplugindir/service.sh stop);
}

# ---------------------------------------
# Set LOX_TIME Enabled / Disabled switch
# ---------------------------------------
if ($LOX_TIME eq "True") {
  $LOX_TIME = "<option value=\"False\">".$No."</option><option value=\"True\" selected>".$Yes."</option>";
} else {
  $LOX_TIME = "<option value=\"False\" selected>".$No."</option><option value=\"True\">".$Yes."</option>";
}

# ---------------------------------------
# Set USE_METRIC Enabled / Disabled switch
# ---------------------------------------
if ($USE_METRIC eq "True") {
  $USE_METRIC = "<option value=\"False\">".$No."</option><option value=\"True\" selected>".$Yes."</option>";
  } else {
    $USE_METRIC = "<option value=\"False\" selected>".$No."</option><option value=\"True\">".$Yes."</option>";
}

# ---------------------------------------
# Set IGNORE_EMPTY Enabled / Disabled switch
# ---------------------------------------
if ($IGNORE_EMPTY eq "True") {
  $IGNORE_EMPTY = "<option value=\"False\">".$No."</option><option value=\"True\" selected>".$Yes."</option>";
  } else {
    $IGNORE_EMPTY = "<option value=\"False\" selected>".$No."</option><option value=\"True\">".$Yes."</option>";
}

# ---------------------------------------
# Set EVAL_VALUES Enabled / Disabled switch
# ---------------------------------------
if ($EVAL_VALUES eq "True") {
  $EVAL_VALUES = "<option value=\"False\">".$No."</option><option value=\"True\" selected>".$Yes."</option>";
  } else {
    $EVAL_VALUES = "<option value=\"False\" selected>".$No."</option><option value=\"True\">".$Yes."</option>";
}

# ---------------------------------------
# Set UDP_MINMAX Enabled / Disabled switch
# ---------------------------------------
if ($UDP_MINMAX eq "True") {
  $UDP_MINMAX = "<option value=\"False\">".$No."</option><option value=\"True\" selected>".$Yes."</option>";
  } else {
    $UDP_MINMAX = "<option value=\"False\" selected>".$No."</option><option value=\"True\">".$Yes."</option>";
}

# ---------------------------------------
# Set FWD_STATUS Enabled / Disabled switch
# ---------------------------------------
if ($FWD_STATUS eq "True") {
  $FWD_STATUS = "<option value=\"False\">".$No."</option><option value=\"True\" selected>".$Yes."</option>";
  } else {
    $FWD_STATUS = "<option value=\"False\" selected>".$No."</option><option value=\"True\">".$Yes."</option>";
}

# ---------------------------------------
# Set FWD_ENABLE Enabled / Disabled switch
# ---------------------------------------
if ($FWD_ENABLE eq "True") {
  $FWD_ENABLE = "<option value=\"False\">".$No."</option><option value=\"True\" selected>".$Yes."</option>";
  } else {
    $FWD_ENABLE = "<option value=\"False\" selected>".$No."</option><option value=\"True\">".$Yes."</option>";
}

# ---------------------------------------
# Fill Miniserver selection dropdown
# ---------------------------------------
if ($lang eq "de") {
  $MSselectlist .= '<option value="">Loxone-Wahl</option>'."\n        ";
} elsif ($lang eq "nl") {
  $MSselectlist .= '<option value="">selecteer Loxone-Server</option>'."\n        ";
} elsif ($lang eq "sk") {
  $MSselectlist .= '<option value="">vyberte Loxone-Server</option>'."\n        ";
} else {
  $MSselectlist .= '<option value="">select Loxone</option>'."\n        ";
}
for (my $i = 1; $i <= $cfg->param('BASE.MINISERVERS');$i++) {
  $MSselectlist .= '        <option value="'.$cfg->param("MINISERVER$i.IPADDRESS").'">'.$cfg->param("MINISERVER$i.IPADDRESS")."</option>";
}

# fill fwdtypelist
my @knowntypes = (
                   ["AMB",          "Ambient Weather-Format (Forward im Ambient-Format via get)",               "Ambient Weather-format - forward in Ambient-protocol via get"],
                   ["APRS",         "APRS-Format etwa für CWOP (Forward via TCP/IP)",                           "APRS-format - forward via TCP/IP to e.g. CWOP"],
                   ["AWEKAS",       "Awekas-Format (Forward im Awekas-API-Format via get)",                     "Awekas-format - forward in Awekas-API-protocol via get"],
                   ["CLIENTRAWTXT", "clientraw.txt-Format (Forward der Daten als WD-Exportdatei)",              "CLIENTRAW-format - forward as WD-export file clientraw.txt"],
                   ["CSV",          "CSV-Format (Forward im Semikolon-separierten CSV-Format via post)",        "CSV-format - forward String with semicolon as separator via post"],
                   ["CSVFILE",      "CSV-Datei - Ausgabe der Daten als CSV-Datei (mit Keys)",                   "CSV-file - output data as CSV file (with keys)"],
                   ["EW",           "Ecowitt-Format (Forward im Ecowitt-Format via post)",                      "Ecowitt-format - forward in Ecowitt-protocol via post"],
                   ["EWUDP",        "Ecowitt-UDP - Versand im Ecowitt-Format via UDP",                          "Ecowitt-UDP - forward as Ecowitt-line via UDP"],
                   ["INFLUXIMP",    "InfluxDB imperial - Versand der imperialen Daten an InfluxDB",             "InfluxDB imperial - send imperial data to InfluxDB"],
                   ["INFLUXMET",    "InfluxDB metrisch - Versand der metrischen Daten an InfluxDB",             "InfluxDB metric - send metric data to InfluxDB"],
                   ["LD",           "LD-Format (Forward für Luftdaten.info-Dienst via post)",                   "Luftdaten.info-format - forward PM-values to Luftdaten.info via post"],
                   ["MQTTIMP",      "MQTT imperial - Versand der imperialen Daten per MQTT",                    "MQTT imperial - send imperial data via MQTT"],
                   ["MQTTMET",      "MQTT metrisch - Versand der metrischen Daten per MQTT",                    "MQTT metric - send metric data via MQTT"],
                   ["MT",           "Meteotemplate-Format (Forward im Meteotemplate-Format via get)",           "Meteotemplate-format - forward in Meteotemplate-protocol via get"],
                   ["RAW",          "RAW-Format (Forward wie Eingangsformat via get)",                          "RAW-format - forward as it comes in via get"],
                   ["RAWAMB",       "RAWAMB-Format (Forward im unveränderten Ambient-Format via get)",          "Ambient-format - forward in untouched Ambient-protocol via get"],
                   ["RAWCSV",       "RAWCSV-Format (Eingangsdaten per post als Semikolon-separiertes CSV)",     "RAWCSV-format - forward input-line separated with semicolon via post"],
                   ["RAWEW",        "RAWEW-Format RAWEW (Forward im unveränderten Ecowitt-Format via post)",    "Ecowitt-format - forward in untouched Ecowitt-protocol via post"],
                   ["RAWTEXT",      "RAWTEXT-Datei - Ausgabe der Eingangsdaten (mit Keys und Zeilenumbruch)",   "RAWTEXT-file - output original input data as TXT file (with keys/line break)"],
                   ["RAWUDP",       "RAWUDP-Format (Forward der Original-Eingangsdaten via UDP)",               "RAWUDP-format - forward original input-line via UDP"],
                   ["REALTIMETXT",  "realtime.txt-Format (Forward der Daten als Cumulus-Exportdatei)",          "REALTIME-format - forward as Cumulus-export file realtime.txt"],
                   ["TXTFILE",      "TXT-Datei Ausgabe der Daten als Text-Datei (mit Keys und Zeilenumbruch)",  "TXT-file - output data as TXT file (with keys and line break)"],
                   ["UDP",          "UDP-Format (Forward der UDP-Ausgabezeile via UDP)",                        "UDP-format - forward as UDP-output via UDP"],
                   ["UDPGET",       "UDPGET-Format (Forward wie UDP-Ausgabezeile via get)",                     "UDPGET-format - forward as UDP-output via get"],
                   ["WC",           "Weathercloud-Format (Forward im Weathercloud-Format via get)",             "Weathercloud-format - forward in Weathercloud-protocol via get"],
                   ["WEATHER365",   "weather365.net-Format (Forward im API-Format für Weather365 via post)",    "weather365.net-format - forward in weather365-API-protocol via post"],
                   ["WETTERCOM",    "wetter.com-Format (Forward im Wetterarchiv-API-Format via get)",           "wetter.com-format - forward in Wetterarchiv-API-protocol via get"],
                   ["WSWIN",        "WSWin - Ablage als WSWin-CSV-Datei wswin.csv für Dateiüberwachung",        "WSWin-format - save as WSWin CSV file wswin.csv for automatic file monitoring"],
                   ["WU",           "WU-Format (Forward im Weather Underground-Format via get)",                "WU-format - forward in Weather Underground-protocol via get"],
                   );
my $lptr;
if ($lang eq "de") { $lptr = 1; } else { $lptr = 2; }
my $selstr;
my $i = 0;
my $spaces = "";
foreach my $fwdtype ( @knowntypes ) {
  if ($FWD_TYPE eq ${$fwdtype}[0]) { $selstr = " selected" } else { $selstr = "" }
  if ($i ne 0) { $spaces = "                  " }
  $fwdtypelist .= $spaces.'<option value="'.${$fwdtype}[0].'"'.$selstr.'>'.${$fwdtype}[$lptr].'</option>'."\n";
  $i++;
}

my $FOSHKonline;
my $FOSHKicon;
my $def_LB_IP;
if ($LB_IP eq "") { $def_LB_IP = $lbip; } else { $def_LB_IP = $LB_IP; }
$FOSHKonline = get("http://".$def_LB_IP.":".$LBH_PORT."/FOSHKplugin/state");
if (defined $FOSHKonline && $FOSHKonline eq "running") {
  $FOSHKicon = "check";
  if ($lang eq "de") {
    $FOSHKonline = "Dienst gestartet";
  } elsif ($lang eq "nl") {
    $FOSHKonline = "Service loopt";
  } elsif ($lang eq "sk") {
    $FOSHKonline = "Služba sa začala";
  } else {
    $FOSHKonline = "Service started";
  }
} else {
  $FOSHKicon = "delete";
  if ($lang eq "de") {
    $FOSHKonline = "Dienst ist nicht aktiv";
  } elsif ($lang eq "nl") {
    $FOSHKonline = "Service is niet actief";
  } elsif ($lang eq "sk") {
    $FOSHKonline = "Služba nie je aktívna";
  } else {
    $FOSHKonline = "Service is not running";
  }
}
$FOSHKrunning = "<input id=\"btnRUN\" data-inline=\"false\" data-mini=\"true\" data-icon=\"".$FOSHKicon."\" type=\"button\" disabled=\"disabled\" name=\"FOSHKrunning\" value=\"".$FOSHKonline."\" />";

if (defined $LOX_IP) { debugPrint("  LOX_IP: ".$LOX_IP); }
if (defined $LOX_PORT) { debugPrint("  LOX_PORT: ".$LOX_PORT); }
if (defined $LB_IP) { debugPrint("  LB_IP: ".$LB_IP); }
if (defined $LBU_PORT) { debugPrint("  LBU_PORT: ".$LBU_PORT); }
if (defined $LBH_PORT) { debugPrint("  LBH_PORT: ".$LBH_PORT); }
if (defined $LOX_TIME) { debugPrint("  LOX_TIME: ".$LOX_TIME); }
if (defined $LBU_PORT) { debugPrint("  LBU_PORT: ".$LBU_PORT); }
if (defined $logfile) { debugPrint("  logfile: ".$logfile); }
if (defined $rawfile) { debugPrint("  rawfile: ".$rawfile); }
if (defined $sndfile) { debugPrint("  sndfile: ".$sndfile); }
if (defined $enabled) { debugPrint("  enabled: ".$enabled); }
if (defined $USE_METRIC) { debugPrint("  USE_METRIC: ".$USE_METRIC); }
if (defined $IGNORE_EMPTY) { debugPrint("  IGNORE_EMPTY: ".$IGNORE_EMPTY); }
if (defined $WS_IP) { debugPrint("  WS_IP: ".$WS_IP); }
if (defined $WS_PORT) { debugPrint("  WS_PORT: ".$WS_PORT); }
if (defined $WS_INTERVAL) { debugPrint("  WS_INTERVAL: ".$WS_INTERVAL); }
if (defined $FWD_URL) { debugPrint("  FWD_URL: ".$FWD_URL); }
if (defined $FWD_TYPE) { debugPrint("  FWD_TYPE: ".$FWD_TYPE); }
if (defined $FWD_SID) { debugPrint("  FWD_SID: ".$FWD_SID); }
if (defined $FWD_PWD) { debugPrint("  FWD_PWD: ".$FWD_PWD); }
if (defined $FWD_STATUS) { debugPrint("  FWD_STATUS: ".$FWD_STATUS); }
if (defined $FWD_ENABLE) { debugPrint("  FWD_ENABLE: ".$FWD_ENABLE); }
if (defined $FWD_IGNORE) { debugPrint("  FWD_IGNORE: ".$FWD_IGNORE); }
if (defined $FWD_INTERVAL) { debugPrint("  FWD_INTERVAL: ".$FWD_INTERVAL); }
if (defined $CSV_NAME) { debugPrint("  CSV_NAME: ".$CSV_NAME); }
if (defined $CSV_INTERVAL) { debugPrint("  CSV_INTERVAL: ".$CSV_INTERVAL); }
if (defined $CSV_FIELDS) { debugPrint("  CSV_FIELDS: ".$CSV_FIELDS); }
if (defined $LBS_PORT) { debugPrint("  LBS_PORT: ".$LBS_PORT); }
if (defined $EVAL_VALUES) { debugPrint("  EVAL_VALUES: ".$EVAL_VALUES); }
if (defined $UDP_MINMAX) { debugPrint("  UDP_MINMAX: ".$UDP_MINMAX); }
if (defined $UDP_STATRESEND) { debugPrint("  UDP_STATRESEND: ".$UDP_STATRESEND); }
if (defined $ADD_ITEMS) { debugPrint("  ADD_ITEMS: ".$ADD_ITEMS); }
debugPrint("parse config file done");

# Init Language
# Clean up lang variable
debugPrint("init language start");
$lang         =~ tr/a-z//cd;
$lang         = substr($lang,0,2);
# If there's no language phrases file for choosed language, use "en" as default
#if (!-e "$installfolder/templates/system/$lang/language.dat") {
#  $lang = "en";
#}
# Read translations / phrases
$languagefile           = "$installfolder/templates/system/$lang/language.dat";
$phrase                 = new Config::Simple($languagefile);
$languagefileplugin     = "$installfolder/templates/plugins/$lbpplugindir/$lang/language.dat";
$phraseplugin           = new Config::Simple($languagefileplugin);
debugPrint("  lang: ".$lang);
debugPrint("init language done");

# Title
$template_title = "$prgname v$prgver";

# ---------------------------------------
# Load header and replace HTML Markup <!--$VARNAME--> with perl variable $VARNAME
# ---------------------------------------
debugPrint("load header file start");
open(F,"$installfolder/templates/system/$lang/header.html") || open(F,"$installfolder/templates/system/en/header.html") || die "Missing template system/$lang/header.html";
while (<F>) {
  $_ =~ s/<!--\$(.*?)-->/${$1}/g;
  print $_;
}
close(F);
debugPrint("load header file done");

# ---------------------------------------
# Load content from template
# ---------------------------------------
debugPrint("load content file start");
#open(F,"$installfolder/templates/plugins/$lbpplugindir/$lang/content.html") || die "Missing template $lang/content.html";
open(F,"$installfolder/templates/plugins/$lbpplugindir/$lang/content.html") || open(F,"$installfolder/templates/plugins/$lbpplugindir/en/content.html") || die "Missing template $lang/content.html";
while (<F>) {
  $_ =~ s/<!--\$(.*?)-->/${$1}/g;
  print $_;
}
close(F);
debugPrint("load content file done");

# ---------------------------------------
# Load footer and replace HTML Markup <!--$VARNAME--> with perl variable $VARNAME
# ---------------------------------------
debugPrint("load footer file start");
open(F,"$installfolder/templates/system/$lang/footer.html") || open(F,"$installfolder/templates/system/en/footer.html") || die "Missing template system/$lang/header.html";
while (<F>) {
  $_ =~ s/<!--\$(.*?)-->/${$1}/g;
  print $_;
}
close(F);
debugPrint("load footer file done");

debugPrint("cgi exit");
exit;
